function [b,a,l2error] = mpiir_l2(M,N,om,D,W,r,aO)
% MPIIR_L2: [b,a,12error] = mpiir_12(M,H,om,D,W,r,aO)
% Least Squares Digital IIR Filter Design with Arbitrary Magnitude
% and Phase Responses and Specified Maximum Pole Radius
%
% OUTPUT:
% b       numerator polynomial coefficients
% a       denominator polynomial coefficients
% 12error  approximation error
% INPUT:
% M       order of numerator polynomial
% N       order of denominator polynomial
% om      frequency grid, 0<=om<=pi
% D       complex desired frequency response on the grid om
% W       positive weighting function on the grid om
% r       maximum pole radius
% a0      initial denominator guess; optional
% EXAMPLE:
% Lowpass filter with approximately linear passband phase
% (passband group delay = 19 samples, maximum pole radius = 0.97)
% om=pi*[linspace(0,.2,20),linspace(.25,1,75)];
% D=[exp(-j*om(1:20)*19),zeros(1,75)];
% W=[ones(1,20),100*ones(1,75)];
% [b,a,e]=mpiir_l2(16,6,om,D,W,.97);
% Author: Mathias C. Lang, Vienna University of Technology, Oct. 98

om=om(:); D=D(:); W=W(:); srW = sqrt(W);
EM = exp(-j*om*(0:max([M,N]))); tol = 1e-4; alpha = 0.5;
disp(['   ERROR    MAX.RADIUS    STEP       SLOPE', 10]);

% initial solution
ini=0;
if nargin == 7,
	a=aO(:);
	if length(a)~=N+1, ini=1;
	elseif a(1)==O, ini=1;
	elseif max(abs(roots(a))) > r, ini=1;
	else
		if a(1)~=1, a=a/a(1); end
		A = freqz(a,1,om); b = lslevin(M+1,om,A.*D,W./(abs(A).^2));
	end
else, ini=1;
end
if ini,    % compute FIR solution
    a = [1;zeros(N,1)]; b = lslevin(M+1,om,D,W);
end
x = [a(2:N+1);b]; delta = [];

% iterate (outer loop)
while 1,
    
	% compute complex error, Jacobian, and objective function value
	A = EM(:,1:N+1)*a; B = EM(:, 1:M+1)*b; H = B./A;
	E = srW.*(D  - H);   l2error  = E'*E;   if N<1,   break;   end
	vec1   =   srW./A;   vec2  =  -vec1.*H;
	J =   [vec2(:,ones(1,N)).*EM(:,2:N+1),vec1(:,ones(1,M+1)).*EM(:,1:M+1)];
    
	% compute search direction
	delta0 = delta;
	[delta,how] = update(J,E,a,r,M,N);
%     delta
%     how
% 	if ~strcmp(how,'ok'), delta=delta0; break; end
	if how<=0, delta=delta0; break; end
    
	% update  solution
	x = x  + alpha*delta;   a =   [1;x(1:N)];   b = x(N+1 :M+N+1);
    
	% display  results
	step = norm(delta)/norm(x);   pr  = max(abs(roots(a)));
	slope  = -2*real(E'*J)*delta/l2error;
	disp([sprintf('\b \b %5.3e %5.3e %5.3e %5.3e ' ,l2error,pr,step,slope), 10])
    
	% check stopping criterion
%     pr
	if (step < tol & abs(slope) < tol) | pr > r, break; end
% 	if (step < tol & abs(slope) < tol), break; end
    
end  % outer loop

disp(['Filter calculated successfully', 10])